from PyQt5.QtWidgets import QGridLayout, QHBoxLayout, QLabel, QMainWindow, QPushButton, QScrollArea, QSizePolicy, QWidget, QLineEdit
from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtCore import Qt, QSize
from PyQt5 import QtWidgets, uic
import sys
import plotly
import plotly.express as px
import plotly.graph_objects as go
import numpy as np
import pandas as pd

from listclass import ObjectLists
from vector3Dclass import Vector3D
from pointclass import Point
from lineclass import Line
from planeclass import Plane
from nameAssignclass import NameAssign
from colorAssignclass import ColorAssign
from solvers import Solvers
from objectsToFig import compileFig
from calcInputclass import CalcInput


# Pre-Inputted Stuff for Testing purposes...
vec = Vector3D(1, 2, 3,append=True)
vec2 = Vector3D(2, 3, 4,append=True)
point = Point(3,3,3,color=(50,50,50),append=True)
line = Line(vec,Vector3D(10,1,1),append=True)
plane = Plane.normalForm(vec,Vector3D(-5,-1,-1,"Herbert",(0,0,0),show=False,append=True),show=True,append=True)
plane2 = Plane.parameterForm(Vector3D(0,0,10),Vector3D(0,1,0),Vector3D(1,0,0),show=True,append=True)
# point2 = Solvers.solveForPointPlane(line,plane2)
# ObjectLists.appendObjDict({point2.getID(): point2})


class MainWindow(QMainWindow):

    def __init__(self):

        super(MainWindow, self).__init__()
        self.title = "Vektorplotter"
        self.setWindowTitle(self.title)
        self.x = 200
        self.y = 100
        self.width = 1000
        self.height = 700
        self.setGeometry(self.x, self.y, self.width, self.height)
        self.mainWidget = QWidget()
        self.mainLayout = QGridLayout()
        self.mainLayout.setColumnStretch(1, 2)
        self.mainWidget.setLayout(self.mainLayout)
        self.setCentralWidget(self.mainWidget)
        self.main()

    def main(self):
        figure = compileFig()
        self.makeWebEngineView(figure)

        self.makeMenuView()
        self.makeListView()
        self.makeNewObjectView()

    def makeWebEngineView(self, fig):
        self.webBox = QWidget()
        self.webBoxLayout = QHBoxLayout()
        self.webBox.setMinimumSize(500, 500)
        self.webBox.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.webBoxLayout.setContentsMargins(0, 0, 0, 0)

        self.html = fig.to_html(include_plotlyjs="cdn", full_html=True, include_mathjax="cdn")

        self.plot_widget = QWebEngineView()
        self.plot_widget.setHtml(self.html)
        self.webBoxLayout.addWidget(self.plot_widget)

        self.webBox.setLayout(self.webBoxLayout)
        self.mainLayout.addWidget(self.webBox, 1, 1, 2, 1)

    def makeListView(self):
        self.listBox = QWidget()
        self.listBoxLayout = QGridLayout()
        self.listBoxLayout.setSpacing(10)
        self.listBox.setMaximumWidth(450)
        self.listBox.setMinimumWidth(450)
        self.listBox.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.listBoxLayout.setContentsMargins(0, 10, 0, 0)

        self.listScroll = QScrollArea()
        self.listScroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.listScroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.listScroll.setWidgetResizable(True)
        self.listScroll.setMaximumWidth(450)
        self.listScroll.setMinimumWidth(450)
        self.listScroll.setWidget(self.listBox)

        self.listLabel = QLabel(self.listBox)
        self.listLabel.setText("List:")
        self.listLabel.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.listBoxLayout.addWidget(self.listLabel, 0, 0, Qt.AlignmentFlag.AlignTop)

        self.objButtonList = []
        self.delObjectButtonList = []
        for element, index in zip(ObjectLists.getObjDict(), range(ObjectLists.getObjDictLen())):
            self.objButtonList.append(QPushButton(self.listBox))
            self.objButtonList[-1].setText(str(element) + ": " + str(ObjectLists.getObjDict()[element]))
            self.objButtonList[-1].adjustSize()
            self.objButtonList[-1].clicked.connect(lambda e = element, i = index: self.highlightObject(e, i))
            self.objButtonList[-1].setMinimumSize(QSize(50, 50))
            self.objButtonList[-1].setMaximumWidth(500)
            
            self.delObjectButtonList.append(QPushButton(self.listBox))
            self.delObjectButtonList[-1].setText("X")
            self.delObjectButtonList[-1].adjustSize()
            self.delObjectButtonList[-1].clicked.connect(lambda e = element, i = index: self.delObject(e, i))
            self.delObjectButtonList[-1].setMinimumSize(QSize(50, 50))
            self.delObjectButtonList[-1].setMaximumWidth(50)

            self.listBoxLayout.addWidget(self.objButtonList[-1], index + 1, 0, Qt.AlignmentFlag.AlignTop)
            self.listBoxLayout.addWidget(self.delObjectButtonList[-1], index + 1, 1, Qt.AlignmentFlag.AlignTop)

        self.listBox.setLayout(self.listBoxLayout)
        self.mainLayout.addWidget(self.listScroll, 1, 0)


    def makeMenuView(self):
        self.menuBox = QWidget()
        self.menuBoxLayout = QHBoxLayout()
        self.menuBoxLayout.setContentsMargins(0, 0, 0, 0)
        self.menuBoxLayout.setSpacing(10)

        self.homeButton = QPushButton()
        self.homeButton.setText("Home")
        self.homeButton.setMaximumSize(75, 30)
        self.homeButton.setMinimumSize(75, 30)
        self.homeButton.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.homeButton.clicked.connect(lambda: self.home()) # home is Placeholder, lambda for future args
        self.menuBoxLayout.addWidget(self.homeButton, 0, Qt.AlignmentFlag.AlignLeft)

        self.otherButton = QPushButton()
        self.otherButton.setText("Test")
        self.otherButton.setMaximumSize(75, 30)
        self.otherButton.setMinimumSize(75, 30)
        self.otherButton.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.otherButton.clicked.connect(lambda: self.home()) # home is Placeholder, lambda for future args
        self.menuBoxLayout.addWidget(self.otherButton, 1, Qt.AlignmentFlag.AlignLeft)

        self.menuBox.setLayout(self.menuBoxLayout)
        self.mainLayout.addWidget(self.menuBox, 0, 0)

    def makeNewObjectView(self):
        self.newBox = QWidget()
        self.newBox.setMaximumSize(450, 50)
        self.newBox.setMinimumSize(450, 50)
        self.newBoxLayout = QHBoxLayout()
        self.newBoxLayout.setContentsMargins(0, 0, 0, 10)

        self.newObjectButton = QPushButton()
        self.newObjectButton.setText("Neues Objekt")
        self.newObjectButton.setMaximumSize(220, 50)
        self.newObjectButton.setMinimumSize(220, 50)
        self.newObjectButton.clicked.connect(lambda: self.newObjectButtonClicked())
        self.newBoxLayout.addWidget(self.newObjectButton)
        
        self.newCalcButton = QPushButton()
        self.newCalcButton.setText("Neue Rechnung")
        self.newCalcButton.setMaximumSize(225, 50)
        self.newCalcButton.setMinimumSize(225, 50)
        self.newCalcButton.clicked.connect(lambda: self.newCalcButtonClicked())
        self.newBoxLayout.addWidget(self.newCalcButton)

        self.newBox.setLayout(self.newBoxLayout)
        self.mainLayout.addWidget(self.newBox, 2, 0)

    def highlightObject(self, elements, index):
        pass

    def delObject(self, element, index):
        if index < ObjectLists.getObjDictLen():
            e = str(list(ObjectLists.getObjDict().keys())[index])
            ObjectLists.removeFromObjDict(e)
            self.listBoxLayout.update()
            self.main()
        else:
            pass

    def home(self):
        pass #Placeholder

    def newObjectButtonClicked(self):
        # this method is called, if the newObjectButton is clicked
        self.showingNewObjectInputLine = False
        if not(self.showingNewObjectInputLine):
            self.showingNewObjectInputLine = True

            self.newObjectInputLine = QLineEdit()
            self.newObjectInputLine.setFrame(True)
            self.listBoxLayout.addWidget(self.newObjectInputLine, ObjectLists.getObjDictLen() + 2, 0, 1, 2)
            self.listBoxLayout.update()
            self.newObjectInputLine.returnPressed.connect(lambda: self.newObjectInput())
        else:
            pass

    def newCalcButtonClicked(self):
        # this method is called, if the newCalcButton is clicked
        self.showingNewCalcInputLine = False
        if not(self.showingNewCalcInputLine):
            self.showingNewCalcInputLine = True

            self.newCalcInputLine = QLineEdit()
            self.newCalcInputLine.setFrame(True)
            self.listBoxLayout.addWidget(self.newCalcInputLine, ObjectLists.getObjDictLen() + 2, 0, 1, 2)
            self.listBoxLayout.update()
            self.newCalcInputLine.returnPressed.connect(lambda: self.newCalcInput())
        else:
            pass

    def newObjectInput(self):
        # this method is called if the return key is pressed in the newObjectButtonClicked method
        self.showingNewObjectInputLine = False
        self.newObjectInputLineText = self.newObjectInputLine.text()
        self.newObjectInputLine.setVisible(False)
        self.listBoxLayout.removeWidget(self.newObjectInputLine)
        self.listBoxLayout.update()
        exec("self.newObject = " + self.newObjectInputLineText)
        ObjectLists.appendObjDict({self.newObject.getID(): self.newObject})
        self.main()

    def newCalcInput(self):
        # this method is called, if the return key is pressed in the newCalcButtonClicked method
        self.showingNewCalcInputLine = False
        self.newCalcInputLineText = self.newCalcInputLine.text()
        self.newCalcInputLine.setVisible(False)
        self.listBoxLayout.removeWidget(self.newCalcInputLine)
        self.listBoxLayout.update()

        CalcInput.handleInput(self.newCalcInputLineText)
        self.main()






from listclass import ObjectLists
from colorAssignclass import ColorAssign
from nameAssignclass import NameAssign


class Point:
    __idTag = "poi"
    __idCount = 0

    def __init__(self, x=0, y=0, z=0, name=None, color=None, show=True, append=False):
        self.__x = float(x)
        self.__y = float(y)
        self.__z = float(z)
        self.show = show
        Point.__idCount += 1
        self.__id = str(self.__idTag+str(self.__idCount))
        if name is None:
            self.__name = NameAssign.getNewName()
        else:
            self.__name = name
        if color is None:
            self.__color = ColorAssign.getNewColor()
        else:
            self.__color = color
        if append:
            ObjectLists.appendObjDict({str(self.__id): self})
            ObjectLists.appendPoiList(self)


    def __del__(self):
        ColorAssign.removeColor(self.__color)
        NameAssign.removeName(self.__name)
        try:
            ObjectLists.removeFromPoiList(self)
            ObjectLists.removeFromObjDict(str(self.__id))
        except ValueError:
            pass

    def __str__(self):
        """Str method for testing purposes.
        Uses given Point instance, returns string."""
        return "("+str(self.__x)+"; "+str(self.__y)+"; "+str(self.__z)+")"

    def getX(self):
        """The get method for the x-komponent.
        Returns x as float."""
        return float(self.__x)

    def getY(self):
        """The get method for the y-komponent.
        Returns y as float."""
        return float(self.__y)

    def getZ(self):
        """The get method for the z-komponent.
        Returns z as float."""
        return float(self.__z)

    def getName(self):
        """The get method for the name.
        Returns name as string."""
        return str(self.__name)

    def getID(self):
        """The get method for the ID.
        Returns ID as string."""
        return str(self.__id)

    def getColor(self):
        """The get method for the color.
        Returns color as tuple of format:
        (Red, Green, Blue).
        Valuerange = 0 to 256"""
        return self.__color

    def setX(self, x=0):
        """The set method for the x-komponent.
        Takes x as float."""
        self.__x = float(x)

    def setY(self, y=0):
        """The set method for the y-komponent.
        Takes y as float."""
        self.__y = float(y)

    def setZ(self, z=0):
        """The set method for the z-komponent.
        Takes z as float."""
        self.__z = float(z)

    def setName(self, name):
        """The set method for the name.
        Takes name as string."""
        self.__name = str(name)

    def setColor(self, color):
        """The set method for the colour.
        Takes colour as tuple of format:
        (Red, Green, Blue).
        Valuerange = 0 to 256"""
        self.__color = color

    x = property(getX, setX)
    y = property(getY, setY)
    z = property(getZ, setZ)
    name = property(getName, setName)



import math
from listclass import ObjectLists
from nameAssignclass import NameAssign
from colorAssignclass import ColorAssign


class Vector3D:
    __idTag = "vec"
    __idCount = 0

    def __init__(self,
                 x=0.0,
                 y=0.0,
                 z=0.0,
                 name=None,
                 color=None,
                 show=True,
                 append=False
                 ):
        """The init function for the Vector3D class.
        Takes x, y and z values as int or float.
        Name has to be a string."""
        self.__x = float(x)
        self.__y = float(y)
        self.__z = float(z)
        self.show = show
        if name is None:
            self.__name = NameAssign.getNewName()
        else:
            self.__name = name
        if color is None:
            self.__color = ColorAssign.getNewColor()
        else:
            self.__color = color
        Vector3D.__idCount += 1
        self.__id = str(self.__idTag+str(self.__idCount))
        if append:
            ObjectLists.appendObjDict({str(self.__id) : self})
            ObjectLists.appendVecList(self)
            


    def __del__(self):
        ColorAssign.removeColor(self.__color)
        NameAssign.removeName(self.__name)
        try:
            ObjectLists.removeFromVecList(self)
            ObjectLists.removeFromObjDict(str(self.__id))
        except ValueError:
            pass

    def getX(self):
        """The get method for the x-komponent.
        Returns x as float."""
        return float(self.__x)

    def getY(self):
        """The get method for the y-komponent.
        Returns y as float."""
        return float(self.__y)

    def getZ(self):
        """The get method for the z-komponent.
        Returns z as float."""
        return float(self.__z)

    def getName(self):
        """The get method for the name.
        Returns name as string."""
        return str(self.__name)

    def getID(self):
        """The get method for the ID.
        Returns ID as string."""
        return str(self.__id)

    def getColor(self):
        """The get method for the color.
        Returns color as tuple of format:
        (Red, Green, Blue).
        Valuerange = 0 to 256"""
        return self.__color

    def setX(self, x=0):
        """The set method for the x-komponent.
        Takes x as float."""
        self.__x = float(x)

    def setY(self, y=0):
        """The set method for the y-komponent.
        Takes y as float."""
        self.__y = float(y)

    def setZ(self, z=0):
        """The set method for the z-komponent.
        Takes z as float."""
        self.__z = float(z)

    def setName(self, name):
        """The set method for the name.
        Takes name as string."""
        self.__name = str(name)

    def setColor(self, color):
        """The set method for the colour.
        Takes colour as tuple of format:
        (Red, Green, Blue).
        Valuerange = 0 to 256"""
        self.__color = color

    def add(self, other):
        """Returns the added vector of two given Vector3D instances
        as a new Vector3D instance."""
        return Vector3D(self.__x+other.__x,
                        self.__y+other.__y,
                        self.__z+other.__z)

    def subtract(self, other):
        """Returns the subtracted vector of two given Vector3D instances as
        a new Vector3D instance."""
        return Vector3D(self.__x-other.__x,
                        self.__y-other.__y,
                        self.__z-other.__z)

    def scalarProduct(self, other):
        """Returns the scalar product (defined as: length(a)*length(b)*cos(a,b))
        of two given Vector3D instances as a scalar (float)."""
        return float(self.__x*other.__x+self.__y*other.__y+self.__z*other.__z)

    def square(self):
        """Returns the scalar Product of a Vector3D with itself."""
        return self.scalarProduct(self)

    def scalarMultiplication(self, scalar):
        """Scalar multiplication (elongation by scalar) of
        a given Vector3D instance and a scalar (int or float).
        Returns new Vector3D instance."""
        scalar = float(scalar)
        return Vector3D(self.__x*scalar,
                        self.__y*scalar,
                        self.__z*scalar)

    def scalarDivision(self, scalar):
        """Returns the scalar devision (shortening by scalar) of
        a given Vector3D instance and a scalar (int or float).
        Returns new Vector3D instance."""
        scalar = float(scalar)
        scalar = 1/scalar
        return Vector3D(self.__x*scalar,
                        self.__y*scalar,
                        self.__z*scalar)

    def vectorProduct(self, other):
        """Returns the vector product of two given Vector3D instances
        as a new Vector3D instance."""
        x = ((self.__y * other.__z) - (self.__z * other.__y))
        y = ((self.__z * other.__x) - (self.__x * other.__z))
        z = ((self.__x * other.__y) - (self.__y * other.__x))
        return Vector3D(x, y, z)

    def length(self):
        """Returns the lenght of the given Vector3D instance."""
        return math.sqrt(self.__x**2+self.__y**2+self.__z**2)

    def __str__(self):
        """Str method for testing purposes.
        Uses given Vector3D instance, returns string."""
        return "("+str(self.__x)+"; "+str(self.__y)+"; "+str(self.__z)+")"

    x = property(getX, setX)
    y = property(getY, setY)
    z = property(getZ, setZ)
    name = property(getName, setName)



import math
import random
# import matplotlib.pyplot as plt  # Needed only for displaying of colors using included testing code


class ColorAssign:
    __colorList = []
    __maxValues = (255, 255, 255)
    __minValues = (0, 0, 0)
    __protectionEnvironment = 50
    __illegalColors = [(0, 0, 0), (255, 255, 255)]

    def __init__(self):
        """placeholder; not necessary"""
        pass

    @classmethod
    def getNewColor(self):
        """returns a new, not-yet-used color"""
        foundValidSeed = False
        newSeed = None
        newColor = None
        while not foundValidSeed:  # Searching for valid seed
            newSeed = (random.randrange(self.__minValues[0],self.__maxValues[0]+1),
                       random.randrange(self.__minValues[1],self.__maxValues[1]+1),
                       random.randrange(self.__minValues[2],self.__maxValues[2]+1)
                       )
            if len(self.__colorList+self.__illegalColors) == 0:  # if: no illegal colors or normal colors
                foundValidSeed = True  # every seed is valid
                newColor = newSeed  # no other colors, so no distance optimasation nescessary
            elif self.evaluateColor(newSeed) != 0:  # if: seed is valid
                foundValidSeed = True
                newColor = self.discreteSpaceHillClimbing(newSeed)  # optimisation: maximising distance to other colors
        self.__colorList.append(newColor)
        return newColor

    @classmethod
    def changeColor(self,
                    objekt,
                    desiredColor):
        """Function to change color of an objekt. Takes desiredColor as tuple of ints: (r,g,b).
           The objekt passed needs a setColor(self,color) and getColor(self) method"""
        oldColor = objekt.getColor()
        objekt.setColor(desiredColor)
        self.removeColor(oldColor)
        self.__colorList.append(desiredColor)

    @classmethod
    def removeColor(self,
                    color):
        """Removes given color from self.__colorList.
           Takes color as tuple of ints: (r,g,b)."""
        if color in self.__colorList:  # check if color is even in list
            self.__colorList.remove(color)

    @classmethod
    def colorDistance(self,
                      color1,
                      color2):
        """Returns the pythagorian distance between 2 colors in 3D R,G,B space as Float.
           Colors are taken as tuples of 3 ints: (r,g,b)
           Uses pythagorian distance in 3dimensions (r,g,b)"""
        return math.sqrt((color1[0]-color2[0])**2+(color1[1]-color2[1])**2+(color1[2]-color2[2])**2)

    @classmethod
    def evaluateColor(self,
                      color):
        """Function used to evaluate a color based on distances to other colors.
           Takes color as tuple of three ints: (r,g,b)"""
        evaluationOutput = 0
        illegal = False
        for illegalColor in self.__illegalColors:
            if self.colorDistance(color, illegalColor) <= self.__protectionEnvironment:
                illegal = True  # enforce reqired minimum distance to illegalColors
        if not illegal:
            evaluationOutput = 200
            for col in self.__illegalColors+self.__colorList:
                evaluationOutput -= 1/self.colorDistance(color,col)**2
        return (1/len(self.__illegalColors+self.__colorList))*evaluationOutput

    @classmethod
    def adjacentColors(self,
                       color):
        """Returns a list of the adjacent colors of a color.
           Takes color as tuble of ints: (r,g,b)."""
        step = 5  # Standard: 5 (increase for faster optimization; decrease for more exact optimization)
        listOfAdjacentColors = [
            (color[0]+step,color[1]-step,color[2]+step),  # Erst einmal nur die acht Ecken des "Würfels"
            (color[0]+step,color[1]+step,color[2]+step),  # um die eingegebene color.
            (color[0]-step,color[1]+step,color[2]+step),  # Mehr Positionen auf diesem "Würfel" wären möglich,
            (color[0]-step,color[1]-step,color[2]+step),  # verlangsamen aber das optimieren.
            (color[0]+step,color[1]-step,color[2]-step),
            (color[0]+step,color[1]+step,color[2]-step),
            (color[0]-step,color[1]+step,color[2]-step),
            (color[0]-step,color[1]-step,color[2]-step)
            ]
        for adjacentColor, index in zip(listOfAdjacentColors,range(0,9)):  # ersetzen aller Farben außerhalb boundaries
            if adjacentColor[0] >= self.__maxValues[0] or adjacentColor[1] >= self.__maxValues[1] or adjacentColor[2] >= self.__maxValues[2] or adjacentColor[0] <= self.__minValues[0] or adjacentColor[1] <= self.__minValues[1] or adjacentColor[2] <= self.__minValues[2]:
                listOfAdjacentColors[index] = color
        return listOfAdjacentColors

    @classmethod
    def discreteSpaceHillClimbing(self,
                                  startColor):
        """Does a Discrete Space Hill Climb Algorithm for Gradient Ascent.
           Finds a lokal maximum of evaluateColor Function, ascending from startColor.
           Takes startColor as tuple of ints: (r, g, b).
           View "https://en.wikipedia.org/wiki/Hill_climbing" for infos on Algorithm.
           The pseudocode found there on [11.02.2021] is the basis of this implementation."""
        currentColor = startColor
        topReached = False
        topColor = None
        while not topReached:
            # print(currentColor)#for testing purposes
            listOfAdjacentColors = self.adjacentColors(currentColor)
            nextEvaluation = 0
            nextColor = None
            for adjacentColor in listOfAdjacentColors:
                if (newEvaluation := self.evaluateColor(adjacentColor)) > nextEvaluation:
                    nextColor = adjacentColor
                    nextEvaluation = newEvaluation  # evaluateColor(adjacentColor)
            if nextEvaluation <= self.evaluateColor(currentColor):
                topReached = True
                topColor = currentColor
            currentColor = nextColor
        return topColor

    @classmethod
    def addColor(self,
                 color):
        """Adds given Color to __colorList.
           Takes color as tuple of ints: (r,g,b)."""
        self.__colorList.append(color)

    @classmethod
    def getColorList(self):
        """Returns a copy of the otherwise internal __colorList"""
        return list(self.__colorList)

######Test Code (requires matplotlib.pyplot as plt, see top)
##colAss = ColorAssign()
####colAss.addColor((255,0,0))#ein paar farben vordefinieren um zu sehen wie gut optimiert wird
####colAss.addColor((0,255,0))
####colAss.addColor((0,0,255))
####colAss.addColor((255,255,0))
####colAss.addColor((0,255,255))
####colAss.addColor((255,0,255))
####colAss.addColor((0,0,0))
##count = 0
##count += len(colAss.getColorList())
##while True:
##    count += 1
##    print("NewColor = "+str(colAss.getNewColor()))
##    x = input()
##    if x == "p":
##        plt.figure(figsize=(count,1))
##        colorList = colAss.getColorList()
##        for n in range(count):
##          ax = plt.subplot(1,count,n+1)
##          plt.imshow([[colorList[n]]])#[[[colorList[n][0],colorList[n][0],[colorList[n][1],colorList[n][1]],[colorList[n][2],colorList[n][2]]]])
##          plt.gray()
##          ax.get_xaxis().set_visible(False)
##          ax.get_yaxis().set_visible(False)
##        plt.show()



import numpy as np
import pandas as pd
import plotly
import plotly.graph_objects as go
import plotly.express as px


from colorAssignclass import ColorAssign
from lineclass import Line
from listclass import ObjectLists
from nameAssignclass import NameAssign
from planeclass import Plane
from pointclass import Point
from solvers import Solvers
from vector3Dclass import Vector3D


def compileFig():
    dictionary = ObjectLists.getObjDict()
    fig = go.Figure()
    # Koordinatenachsen
    fig.add_trace(go.Scatter3d(x=[-50, 50], y=[0, 0], z=[0, 0], mode="lines", line_color="rgb(0,0,0)", name="x"))
    fig.add_trace(go.Scatter3d(x=[0, 0], y=[-50, 50], z=[0, 0], mode="lines", line_color="rgb(0,0,0)", name="y"))
    fig.add_trace(go.Scatter3d(x=[0, 0], y=[0, 0], z=[-50, 50], mode="lines", line_color="rgb(0,0,0)", name="z"))

    # Hinzugefügte Objekte
    for elem in dictionary:
        if elem[0:3] == "vec":
            elem = dictionary[elem]
            if elem.show is True:
                fig.add_trace(go.Scatter3d(x=[0, elem.x], y=[0, elem.y], z=[0, elem.z], mode="lines", line_color="rgb"+str(elem.getColor()), name=elem.getID()))
        elif elem[0:3] == "lin":
            elem = dictionary[elem]
            if elem.show is True:
                borders = [
                    Plane.normalForm(Vector3D(-50,0,0,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(1,0,0,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0)),
                    Plane.normalForm(Vector3D(50,0,0,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(1,0,0,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0)),
                    Plane.normalForm(Vector3D(0,-50,0,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(0,1,0,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0)),
                    Plane.normalForm(Vector3D(0,50,0,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(0,1,0,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0)),
                    Plane.normalForm(Vector3D(0,0,50,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(0,0,1,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0)),
                    Plane.normalForm(Vector3D(0,0,-50,show=False,append=False,color=(0,0,0)),normalVector=Vector3D(0,0,1,show=False,append=False,color=(0,0,0)),show=False,append=False,color=(0,0,0))]
                schnittpunkte = []
                for plane in borders:
                    schnittpunkt = Solvers.solveForPointPlane(elem,plane)
                    if schnittpunkt is not None:
                        schnittpunkte.append(schnittpunkt)
                if len(schnittpunkte) > 2:
                    x1 = schnittpunkte[0].x
                    x2 = schnittpunkte[1].x
                    y1 = schnittpunkte[0].y
                    y2 = schnittpunkte[1].y
                    z1 = schnittpunkte[0].z
                    z2 = schnittpunkte[1].z
                else:
                    x1 = schnittpunkte[0].x
                    x2 = schnittpunkte[1].x
                    y1 = schnittpunkte[0].y
                    y2 = schnittpunkte[1].y
                    z1 = schnittpunkte[0].z
                    z2 = schnittpunkte[1].z
                fig.add_trace(go.Scatter3d(x=[x1, x2], y=[y1, y2], z=[z1, z2], mode="lines", line_color="rgb"+str(elem.getColor()), name=elem.getID()))
        elif elem[0:3] == "pla":
            elem = dictionary[elem]
            if elem.show:
                edges = [
                    Line(Vector3D(50, 50, 50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(50, 0.1, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(50, 50, 50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 50, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(50, 50, 50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 0.1, 50, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, -50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(50, 0.1, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, -50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 50, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, -50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 0.1, 50, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, -50, 50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(50, 0.1, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, -50, 50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 50, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(50, -50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 0.1, 50, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(50, -50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 50, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, 50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(0.1, 0.1, 50, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False),
                    Line(Vector3D(-50, 50, -50, show=False, append=False, color=(0, 0, 0)),
                         Vector3D(50, 0.1, 0.1, show=False, append=False, color=(0, 0, 0)), name="edge", color=(0, 0, 0),
                         show=False, append=False)
                ]
                schnittpunkte = []
                for edge in edges:
                    schnittpunkt = Solvers.solveForPointPlane(edge, elem)
                    if schnittpunkt is not None:
                        schnittpunkte.append(schnittpunkt)
                x = []
                y = []
                z = []
                for schnittpunkt in schnittpunkte:
                    if 51 > schnittpunkt.x > -51 and 51 > schnittpunkt.y > -51 and 51 > schnittpunkt.z > -51:
                        x.append(schnittpunkt.x)
                        y.append(schnittpunkt.y)
                        z.append(schnittpunkt.y)
                fig.add_trace(go.Mesh3d(x=x,y=y,z=z,color="rgb"+str(elem.getColor()),opacity=0.5,name=elem.getID(),showlegend=True))
        elif elem[0:3] == "poi":
            elem = dictionary[elem]
            if elem.show:
                fig.add_trace(go.Scatter3d(x=[elem.x], y=[elem.y], z=[elem.z], line_color="rgb"+str(elem.getColor()), name=elem.getID()))
    # fig.show()
    return fig




from listclass import ObjectLists
from solvers import Solvers
import re


class CalcInput:

    def __init__(self) -> None:
        pass

    @classmethod
    def handleInput(self, input):
        
        if "+" in input:
            self.handleAddition(self, input)
        elif "add" in input:
            pass #self.handleExplicitAddition(self, input)
        elif "-" in input:
            self.handleSubtraction(self, input)
        elif "subtract" in input:
            pass #self.handleExplicitSubtraction(self, input)
        elif "*" in input:
            self.handleScalarProduct(self, input)
        elif "scalarProduct" in input:
            pass #self.handleExplicitScalarProduct(self, input)
        elif ":" in input or "/" in input:
            self.handleDivision(self, input)
        elif "divide" in input:
            pass #self.handleExplicitDivision(self, input)
        elif "x" in input:
            self.handleVectorProduct(self, input)
        elif "kreuz" in input:
            pass #self.handleExplicitVectorProduct(self, input)
        elif "schneiden" in input or "Schneiden" in input:
            self.handleSchneiden(self, input)
        elif "d" in input or "D" in input:
            self.handleDistance(self, input)
        elif "winkel" in input:
            self.handleAngle(self, input)
        else:
            print("nothing to handle!")

        
    def handleAddition(self, input):
        addition = re.split(r"[+]", input)
        if not(len(addition) == 1):
            firstVec = ObjectLists.getObjDict().get(addition[0].strip())
            addedVec = firstVec
            for index in range(len(addition) - 1):
                try:
                    nextVec = ObjectLists.getObjDict().get(addition[index + 1].strip())
                    addedVec = addedVec.add(nextVec)
                except IndexError:
                    pass
                except Exception:
                    pass

            print(str(input) + ": " + str(addedVec)) # Testing Purposes


    def handleExplicitAddition(self, input):
        pass #Future Implementation

    def handleSubtraction(self, input):
        subtraction = re.split(r"[-]", input)
        if not(len(subtraction) == 1):
            firstVec = ObjectLists.getObjDict().get(subtraction[0].strip())
            subtractedVec = firstVec
            for index in range(len(subtraction) - 1):
                try:
                    nextVec = ObjectLists.getObjDict().get(subtraction[index + 1].strip())
                    subtractedVec = subtractedVec.subtract(nextVec)
                except IndexError:
                    pass
                except Exception:
                    pass

            print(str(input) + ": " + str(subtractedVec)) # Testing Purposes

    def handleExplicitSubtraction(self, input):
        pass #Future Implementation

    def handleScalarProduct(self, input):
        scalar = re.split(r"[*]", input)
        if not(len(scalar) == 1):
            firstVec = ObjectLists.getObjDict().get(scalar[0].strip())
            multVec = firstVec
            for index in range(len(scalar) - 1):
                try:
                    nextVec = ObjectLists.getObjDict().get(scalar[index + 1].strip())
                    multNum = multVec.scalarProduct(nextVec)
                    print(str(input) + ": " + str(multNum)) # Testing Purposes
                except IndexError:
                    pass
                except Exception:
                    nextNum = scalar[index + 1].strip()
                    multVec = multVec.scalarMultiplication(nextNum)
                    print(str(input) + ": " + str(multVec)) # Testing Purposes


    def handleExplicitScalarProduct(self, input):
        pass #Future Implementation

    def handleDivision(self, input):
        try:
            division = re.split(r"[:]", input)
        except len(division) == 1:
            division = re.split(r"[/]", input)

        if not(len(division) == 1):
            firstVec = ObjectLists.getObjDict().get(division[0].strip())
            divVec = firstVec
            for index in range(len(division) - 1):
                try:
                    nextNum = division[index + 1].strip()
                    divVec = divVec.scalarDivision(nextNum)
                except IndexError:
                    pass
                except Exception:
                    pass

            print(str(input) + ": " + str(divVec)) # Testing Purposes


    def handleExplicitDivision(self, input):
        pass #Future Implementation


    def handleVectorProduct(self, input):
        product = re.split(r"[x]", input)
        if not(len(product) == 1):
            firstVec = ObjectLists.getObjDict().get(product[0].strip())
            multVec = firstVec
            for index in range(len(product) - 1):
                try:
                    nextVec = ObjectLists.getObjDict().get(product[index + 1].strip())
                    multVec = multVec.vectorProduct(nextVec)
                except IndexError:
                    pass
                except Exception:
                    pass

            print(str(input) + ": " + str(multVec)) # Testing Purposes

    
    def handleExplicitVectorProduct(self, input):
        pass #Future Implementation
    

    def handleSchneiden(self, input):
        elements = re.split(r"[(,)]", input)
        firstElement = elements[1].strip()
        secondElement = elements[2].strip()

        if "lin" in firstElement:
            if "lin" in secondElement:
                point = Solvers.schnittpunkt(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(point)
            elif "pla" in secondElement:
                point = Solvers.solveForPointPlane(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(point)
            else:
                print("No valid arguments for Schneiden.")
        elif "pla" in firstElement:
            if "lin" in secondElement:
                point = Solvers.solveForPointPlane(ObjectLists.getObjDict().get(secondElement), ObjectLists.getObjDict().get(firstElement))
                print(point)
            elif "pla" in secondElement:
                line = Solvers.schnittgerade(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(line)
            else:
                print("No valid arguments for Schneiden.")
        else:
            print("No valid arguments for Schneiden.")


    def handleDistance(self, input):
        elements = re.split(r"[(,)]", input)
        firstElement = elements[1].strip()
        secondElement = elements[2].strip()

        if "poi" in firstElement:
            if "poi" in secondElement:
                distance = Solvers.distancePointPoint(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            elif "lin" in secondElement:
                distance = Solvers.distancePointLine(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            elif "pla" in secondElement:
                distance = Solvers.distancePlanePoint(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            else:
                print("No valid arguments for Distance.")
        elif "lin" in firstElement:
            if "poi" in secondElement:
                distance = Solvers.distancePointLine(ObjectLists.getObjDict().get(secondElement), ObjectLists.getObjDict().get(firstElement))
                print(distance)
            elif "lin" in secondElement:
                distance = Solvers.distanceLineLine(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            elif "pla" in secondElement:
                distance = Solvers.distanceLinePlane(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            else:
                print("No valid arguments for Distance.")
        elif "pla" in firstElement:
            if "poi" in secondElement:
                distance = Solvers.distancePlanePoint(ObjectLists.getObjDict().get(secondElement), ObjectLists.getObjDict().get(firstElement))
                print(distance)
            elif "lin" in secondElement:
                distance = Solvers.distanceLinePlane(ObjectLists.getObjDict().get(secondElement), ObjectLists.getObjDict().get(firstElement))
                print(distance)
            elif "pla" in secondElement:
                distance = Solvers.distancePlanePlane(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
                print(distance)
            else:
                print("No valid arguments for Distance.")
        else:
            print("No valid arguments for Distance.")

    def handleAngle(self, input):
        elements = re.split(r"[(,)]", input)
        firstElement = elements[1].strip()
        secondElement = elements[2].strip()

        try:
            angle = Solvers.solveForSchnittwinkel(ObjectLists.getObjDict().get(firstElement), ObjectLists.getObjDict().get(secondElement))
            print(angle)
        except Exception:
            print("No valid arguments for Angle.")

    



class NameAssign:
    __alphabet = (  # letters used in the composition of new names
        "a", "b", "c", "d", "e", "f", "g", "h", "i",
        "j", "k", "l", "m", "n", "o", "p", "q", "r",
        "s", "t", "u", "v", "w", "x", "y", "z"
        )
    __protectedNames = ["x", "y", "z"]  # names that should not be assigned
    __nameList = []
    __userAssignedNames = []
    __freeNames = []
    __reuseRemovedNames = True  # Standard: True
    __autoAssignedNameCount = 0
    
    def __init__(self):
        """placeholder, not necessary"""
        pass
    
    @classmethod
    def getNewName(self):
        """returns a new, not-yet-used or protected name"""
        newName = ""
        if self.__reuseRemovedNames and len(self.__freeNames) != 0:
            newName = self.__freeNames.pop()
        else:
            foundValidName = False
            while not foundValidName:
                if self.__autoAssignedNameCount >= len(self.__alphabet):
                    newName = self.__alphabet[(self.__autoAssignedNameCount//len(self.__alphabet))-1]+self.__alphabet[self.__autoAssignedNameCount%len(self.__alphabet)]
                else:
                    newName = self.__alphabet[self.__autoAssignedNameCount]
                if not newName in self.__protectedNames and not newName in self.__nameList:
                    foundValidName = True
                self.__autoAssignedNameCount += 1
        self.__nameList.append(newName)
        return newName

    @classmethod
    def changeName(self,
                   objekt,
                   desiredName: str = ""):
        """Function to change name of an objekt. Takes desiredName as string.
           The objekt passed needs a setName(self,name) and getName(self) method"""
        oldName = objekt.getName()
        objekt.setName(desiredName)
        self.removeName(oldName)
        self.__nameList.append(desiredName)
        self.__userAssignedNames.append(desiredName)

    @classmethod
    def protectName(self,
                    nameToProtect: str = ""):
        """Deprotects a name given as string.
           Has no effect on existing Names."""
        self.__protectedNames.append(nameToProtect)

    @classmethod
    def deprotectName(self,
                      nameToDeprotect):
        """Protects a name given as string.
           Has no effect on existing Names."""
        if nameToDeprotect in self.__protectedNames:  # check if name is even in list
            self.__protectedNames.remove(nameToDeprotect)

    @classmethod
    def removeName(self,
                   name):
        """Removes given name from self.__nameList.
           Takes name as string."""
        if name in self.__nameList:  # check if name is even in list
            self.__nameList.remove(name)
            self.__freeNames.append(name)
        if name in self.__userAssignedNames:  # check if name is even in list
            self.__userAssignedNames.remove(name)

    @classmethod   
    def addUserAssignedName(self,
                            name):
        """Adds a name (string) to __userAssignedNames and __nameList.
           Doesn't give any objekt this name."""
        self.__userAssignedNames.append(name)
        self.__nameList.append(name)



import numpy as np
from vector3Dclass import Vector3D
from pointclass import Point
from planeclass import Plane
from lineclass import Line
from colorAssignclass import ColorAssign
from nameAssignclass import NameAssign
from listclass import ObjectLists
import math


class Solvers:
    
    def __init__(self):
        """Placeholder; not necessary."""
        pass

    @classmethod
    def solveForPointPlane(self, line: Line, plane: Plane):
        if plane.getType() == "normal":
            normVec = plane.getNormalVector()
            posVecP = plane.getPositionVector()
            scalarParam = posVecP.scalarProduct(normVec)
            posVec = line.getPositionVector()
            dirVec = line.getDirectionVector()
            xValNoVar = normVec.getX()*posVec.getX()
            yValNoVar = normVec.getY()*posVec.getY()
            zValNoVar = normVec.getZ()*posVec.getZ()
            xValVar = normVec.getX()*dirVec.getX()
            yValVar = normVec.getY()*dirVec.getY()
            zValVar = normVec.getZ()*dirVec.getZ()
            ValVar = xValVar+yValVar+zValVar
            ValNoVar = scalarParam - xValNoVar - yValNoVar - zValNoVar
            Var = ValNoVar/ValVar
            SchnittPoint = Point(posVec.getX()+Var*dirVec.getX(),posVec.getY()+Var*dirVec.getY(),posVec.getZ()+Var*dirVec.getZ(),color=(0,0,0))
            return SchnittPoint

        elif plane.getType() == "coordinate":
            normVec = plane.getNormalVector()
            scalarParam = plane.getScalarParameter()
            posVec = line.getPositionVector()
            dirVec = line.getDirectionVector()
            xValNoVar = normVec.getX()*posVec.getX()
            yValNoVar = normVec.getY()*posVec.getY()
            zValNoVar = normVec.getZ()*posVec.getZ()
            xValVar = normVec.getX()*dirVec.getX()
            yValVar = normVec.getY()*dirVec.getY()
            zValVar = normVec.getZ()*dirVec.getZ()
            ValVar = xValVar+yValVar+zValVar
            ValNoVar = scalarParam - xValNoVar - yValNoVar - zValNoVar
            Var = ValNoVar/ValVar
            SchnittPoint = Point(posVec.getX()+Var*dirVec.getX(),posVec.getY()+Var*dirVec.getY(),posVec.getZ()+Var*dirVec.getZ(),color=(0,0,0))
            return SchnittPoint
        elif plane.getType() == "parameter":
            normVec = plane.getDirectionVectorOne().vectorProduct(plane.getDirectionVectorTwo())
            posVecP = plane.getPositionVector()
            scalarParam = posVecP.scalarProduct(normVec)
            posVec = line.getPositionVector()
            dirVec = line.getDirectionVector()
            xValNoVar = normVec.getX()*posVec.getX()
            yValNoVar = normVec.getY()*posVec.getY()
            zValNoVar = normVec.getZ()*posVec.getZ()
            xValVar = normVec.getX()*dirVec.getX()
            yValVar = normVec.getY()*dirVec.getY()
            zValVar = normVec.getZ()*dirVec.getZ()
            ValVar = xValVar+yValVar+zValVar
            ValNoVar = scalarParam - xValNoVar - yValNoVar - zValNoVar
            Var = ValNoVar/ValVar
            SchnittPoint = Point(posVec.getX()+Var*dirVec.getX(),posVec.getY()+Var*dirVec.getY(),posVec.getZ()+Var*dirVec.getZ(),color=(0,0,0))
            return SchnittPoint

    @classmethod
    def solveForSchnittstelle(self,
                              line1: Line,
                              line2: Line):
        """Solves for the Schnittstelle between two Lines of dtype Line.
           Returns array of floatpoint numbers that are the coefficients for the Schnittstelle
           in the same order as given Lines."""
        coefficientMatrix = [
            [line1.dirVec.x, -line2.dirVec.x],
            [line1.dirVec.y, -line2.dirVec.y],
            [line1.dirVec.z, -line2.dirVec.z]
            ]
        equalMatrix = [
            line2.posVec.x-line1.posVec.x,
            line2.posVec.y-line1.posVec.y,
            line2.posVec.z-line1.posVec.z
            ]
        try:
            answers = np.linalg.solve(
                np.array(
                    [
                        coefficientMatrix[0],
                        coefficientMatrix[1]
                    ]
                    ),
                np.array(
                    [
                        equalMatrix[0],
                        equalMatrix[1]
                    ]
                         )
                )
        except np.linalg.LinAlgError:
            try:
                answers = np.linalg.solve(
                    np.array(
                        [
                            coefficientMatrix[0],
                            coefficientMatrix[2]
                        ]
                        ),
                    np.array(
                        [
                            equalMatrix[0],
                            equalMatrix[2]
                        ]
                             )
                    )
            except np.linalg.LinAlgError:
                try:
                    answers = np.linalg.solve(
                        np.array(
                            [
                                coefficientMatrix[0],
                                coefficientMatrix[1]
                            ]
                            ),
                        np.array(
                            [
                                equalMatrix[0],
                                equalMatrix[1]
                            ]
                                 )
                        )
                except np.linalg.LinAlgError:
                    answers = None
        return answers

    @classmethod
    def solveForSchnittgerade(self,
                              plane1: Plane,
                              plane2: Plane):
        """Use Solvers.schnittgerade()"""
        return self.schnittgerade(plane1, plane2)

    @classmethod
    def schnittpunkt(self,
                     line1: Line,
                     line2: Line):
        """Returns the Schnittpunkt between two Lines.
           Uses Solvers.solveForSchnittstelle.
           Returns a Point instance."""

        posVec1 = line1.getPositionVector()
        posVec2 = line2.getPositionVector()
        dirVec1 = line1.getDirectionVector()
        dirVec2 = line2.getDirectionVector()
        schnittstelle = self.solveForSchnittstelle(line1, line2)
        ValOne = posVec1.add(dirVec1.scalarMultiplication(schnittstelle[0]))
        ValTwo = posVec2.add(dirVec2.scalarMultiplication(schnittstelle[0]))
        if ValOne.getX() == ValTwo.getX() and ValOne.getY() == ValTwo.getY() and ValOne.getZ() == ValTwo.getZ():
            schnittpunkt = ValOne
            schnittpunkt = Point(ValOne.getX(),ValOne.getY(),ValOne.getZ())
        else: 
            schnittpunkt = None   
        return schnittpunkt

    @classmethod
    def schnittgerade(self,
                      plane1: Plane,
                      plane2: Plane):
        """Calculates and Returns the schnittgerade of two Planes.
           Returns a Line instance."""
        pla1 = plane1.convertToHessianNormalForm()
        pla2 = plane2.convertToHessianNormalForm()

        normvec1 = pla1.normVec
        normvec2 = pla2.normVec

        posnum1 = pla1.posVec.scalarProduct(normvec1)
        posnum2 = pla2.posVec.scalarProduct(normvec2)

        schnittgerade = not Solvers.checkLinearAbhaengig(normvec1, normvec2)  # Testing if the Planes are not Parallel

        if schnittgerade:
            dirVec = normvec1.vectorProduct(normvec2)
            posVecScalar1 = ((posnum1*normvec2.square())-(posnum2*normvec1.scalarProduct(normvec2)))/((normvec1.square()*normvec2.square())-normvec1.scalarProduct(normvec2)**2)
            posVecScalar2 = ((posnum2*normvec1.square())-(posnum1*normvec1.scalarProduct(normvec2)))/((normvec1.square()*normvec2.square())-normvec1.scalarProduct(normvec2)**2)
            normvec1 = normvec1.scalarMultiplication(posVecScalar1)
            normvec2 = normvec2.scalarMultiplication(posVecScalar2)
            posVec = normvec1.add(normvec2)
            schnittgerade = Line(posVec, dirVec, name=("Schnittgerade von "+pla1.name+" und "+pla2.name))
        return schnittgerade

    @classmethod
    def checkParallel(self,
                      objekt1,
                      objekt2):
        """Not jet completely integrated.
           Will return False if objects (pair of Line(s) and/or Plane(s)) are not parallel.
           Else returns True."""
        parallel = True
        if type(objekt1) == Line and type(objekt2) == Line:
            if Solvers.solveForSchnittstelle(objekt1, objekt2) is not None:
                parallel = False
        elif type(objekt1) == Line and type(objekt2) == Plane:
            pass  # still requires implementation of Line and Plane Schnittpunkt Calculations
        elif type(objekt1) == Plane and type(objekt2) == Line:
            pass  # see above
        elif type(objekt1) == Plane and type(objekt2) == Plane:
            if Solvers.schnittgerade(objekt1, objekt2) is not False:
                parallel = False
        else:
            print("We cannot check if these objekts are Parallel")
        return parallel

    @classmethod
    def checkPointInLine(self,
                         line: Line,
                         point: Point):
        """Uses Solvers.checkLinearAbhaengig to check if a given point is on a given line.
           Returns True if the point is on the Line.
           Else returns False.
           See Solvers.checkLinearAbhaengig."""
        return self.checkLinearAbhaengig(
            line.dirVec,
            Vector3D(point.x+line.posVec.x, point.y+line.posVec.y, point.z+line.posVec.z))

    @classmethod
    def checkPointInPlane(self, plane:Plane, point: Point):
        vector = Vector3D(point.getX(),point.getY(),point.getZ())
        typeOfPlane = plane.getType()
        pointInPlane = False
        if typeOfPlane == "normal":
           result = vector.subtract(plane.getPositionVector())
           answer = result.scalarProduct(plane.getNormalVector())
           if answer == 0:
               pointInPlane = True
           else:
               pointInPlane = False
        elif typeOfPlane == "parameter":
            dirVec1 = plane.getDirectionVectorOne()
            dirVec2 = plane.getDirectionVectorTwo()
            result = vector.subtract(plane.getPositionVector())
            arrayOne = np.array([[dirVec1.getX(),dirVec1.getY()], [dirVec2.getX(),dirVec2.getY()]])
            arrayTwo = np.array([result.getX(),result.getY()])
            solution = np.linalg.solve(arrayOne,arrayTwo)
            if solution[0]*dirVec1.getZ() + solution[1]*dirVec2.getZ() == result.getZ():
                pointInPlane = True
            else: 
                pointInPlane = False
        elif typeOfPlane == "coordinate":
            normVec = plane.getNormalVector()
            if point.getX()*normVec.getX() + point.getY()*normVec.getY() + point.getZ()*normVec.getZ() == plane.getScalarParameter():
                pointInPlane = True
            else:
                pointInPlane = False
        return pointInPlane

    @classmethod
    def checkColinear(self,
                      obj1,
                      obj2):
        """Still requires implementation."""
        pass

    @classmethod
    def checkLinearAbhaengig(self,
                             vec1: Vector3D,
                             vec2: Vector3D):
        """Tests if two vectors are "linear abhängig".
           Takes two Vector3D instances.
           Returns True if they are linear abhängig.
           Else returns False."""
        coefficientMatrix = np.array([
            [vec1.x],
            [vec1.y],
            [vec1.z]
            ])
        equalMatrix = np.array([
            vec2.x,
            vec2.y,
            vec2.z
            ])
        try:
            answer = np.linalg.solve(
                np.array([coefficientMatrix[0]]),
                np.array([equalMatrix[0]])
                )
        except np.linalg.LinAlgError:
            try:
                answer = np.linalg.solve(
                    np.array([coefficientMatrix[1]]),
                    np.array([equalMatrix[1]])
                    )
            except np.linalg.LinAlgError:
                try:
                    answer = np.linalg.solve(
                        np.array([coefficientMatrix[2]]),
                        np.array([equalMatrix[2]])
                        )
                except np.linalg.LinAlgError:
                    answer = False  # Wenn dieser Punkt erricht wird sind sie nicht Abhängig.
        if answer != False:
            print(answer)
            print(coefficientMatrix)
            coefficientMatrix = np.inner(coefficientMatrix, answer)
            print(coefficientMatrix)
            print(equalMatrix)
            if coefficientMatrix[0] == equalMatrix[0] and coefficientMatrix[1] == equalMatrix[1] and coefficientMatrix[2] == equalMatrix[2]:
                answer = True
            else:
                answer = False
        return answer

    @classmethod
    def solveForSchnittwinkel(self,
                              objekt1,
                              objekt2):
        """Calculates the Schnittwinkel of two Planes or Lines (or a Plane and a Line).
           Takes either Line or Plane instances for objekt1 and objekt2.
           Returns a angle as a floating point number of degrees."""
        winkel = None
        if type(objekt1) == Line and type(objekt2) == Line:
            if Solvers.solveForSchnittstelle(objekt1, objekt2) is not None:
                winkel = math.degrees(math.acos(
                    abs(objekt1.dirVec.scalarProduct(objekt2.dirVec)) /
                    (objekt1.dirVec.length()*objekt2.dirVec.length())))
        elif type(objekt1) == Line and type(objekt2) == Plane:
            winkel = math.degrees(math.asin(
                abs(objekt1.dirVec.scalarProduct(objekt2.convertToHessianNormalForm().normVec)) / (
                    objekt1.dirVec.length())
            ))
        elif type(objekt1) == Plane and type(objekt2) == Line:
            winkel = math.degrees(math.asin(
                abs(objekt2.dirVec.scalarProduct(objekt1.convertToHessianNormalForm().normVec)) / (
                    objekt2.dirVec.length())
            ))
        elif type(objekt1) == Plane and type(objekt2) == Plane:
            winkel = math.degrees(math.acos(
                abs(objekt1.convertToHessianNormalForm().normVec.scalarProduct(
                    objekt2.convertToHessianNormalForm().normVec))
                ))
        else:
            print("We cannot calculate this Angle.")
        return winkel

    @classmethod
    def distancePointPoint(self, point:Point, point1:Point):
        Vec = Vector3D(point.getX()-point1.getX(),point.getY()-point1.getY(),point.getZ()-point1.getZ())
        distance = abs(Vec.length())
        return distance

    @classmethod
    def distanceLineLine(self, line1:Line,line2:Line):
        if Solvers.checkLinearAbhaengig(line1.getDirectionVector(),line2.getDirectionVector()) == True:
            if Solvers.checkPointInLine(line1,line2.getPositionVector()) ==True:
                print("Identisch")
            else:
                distance = Solvers.distancePointLine(line2.getPositionVector(),line1)
        elif Solvers.schnittpunkt(line1,line2) is not None:
            distance = 0
        else:
            normVec = line1.getDirectionVector().vectorProduct(line2.getDirectionVector())
            scalarParam = normVec.scalarProduct(line1.getPositionVector())
            helpPlane = Plane.coordinateForm(normVec,scalarParam)
            distance = Solvers.distancePlanePoint(line2.getPositionVector(),helpPlane)
        return distance


    @classmethod
    def distancePlanePoint(self, point:Point, plane:Plane):
        if Solvers.checkPointInPlane(plane,point) == False:
            if plane.getType() == "coordinate":
                hess = plane.convertToHessianNormalForm()
                normVec = hess.getNormalVector()
                scalarParam = hess.getScalarParameter()
                distance = normVec.getX()*point.getX()+normVec.getY()*point.getY()+normVec.getZ()*point.getZ()-scalarParam
            else:
                hess = plane.convertToHessianNormalForm()
                pointVec = Vector3D(point.getX(),point.getY(),point.getZ())
                subtract = pointVec.subtract(hess.getPositionVector())
                distance = subtract.scalarProduct(hess.getNormalVector())
                distance = abs(distance)
        else:
            distance = 0
        return distance

    @classmethod
    def distanceLinePlane(self, line:Line, plane:Plane):
        if plane.getType() == "normal" or plane.getType() == "coordinate":
            normVec = plane.getNormalVector()
            if normVec.scalarProduct(line.getDirectionVector()) == 0:
                distance = Solvers.distancePlanePoint(line.getPositionVector(), plane)
            else: 
                distance = 0
            return distance
        else: 
            normVec = plane.getDirectionVectorOne().vectorProduct(plane.getDirectionVectorTwo())
            if normVec.scalarProduct(line.getDirectionVector()) == 0:
                distance = Solvers.distancePlanePoint(line.getPositionVector(), plane)
            else:     
                distance = 0
            return distance

    @classmethod
    def distancePointLine(self, point:Point, line:Line):
        helpPlane = Plane.normalForm(Vector3D(point.getX(),point.getY(),point.getZ()),line.getDirectionVector())
        pointSchnitt = Solvers.solveForPointPlane(line,helpPlane)
        schnittPoint = Point(pointSchnitt.getX(),pointSchnitt.getY(),pointSchnitt.getZ())
        distance = Solvers.distancePointPoint(schnittPoint,point)
        return distance









class ObjectLists:

    __objDict = {}
    __vecList = []
    __poiList = []
    __linList = []
    __plaList = []

    def __init__(self) -> None:
        """Platzhalter... Ist z. Z. nicht nötig."""
        pass

    @classmethod
    def getObjDict(self):
        """Returns ObjectLists.__objDict."""
        return self.__objDict

    @classmethod
    def getVecList(self):
        """Returns ObjectLists.__vecList."""
        return self.__vecList

    @classmethod
    def getPoiList(self):
        """Returns ObjectLists.__poiList."""
        return self.__poiList

    @classmethod
    def getLinList(self):
        """Returns ObjectLists.__linList."""
        return self.__linList

    @classmethod
    def getPlaList(self):
        """Returns ObjectLists.__plaList."""
        return self.__plaList

    @classmethod
    def getObjDictLen(self):
        """Returns integer length of ObjectLists.__objDict."""
        return len(self.__objDict)

    @classmethod
    def getVecListLen(self):
        return len(self.__vecList)

    @classmethod
    def getPoiListLen(self):
        return len(self.__poiList)

    @classmethod
    def getLinListLen(self):
        return len(self.__linList)

    @classmethod
    def getPlaListLen(self):
        return len(self.__plaList)

    @classmethod
    def appendObjDict(self, x):
        self.__objDict.update(x)

    @classmethod
    def appendVecList(self, x):
        self.__vecList.append(x)

    @classmethod
    def appendPoiList(self, x):
        self.__poiList.append(x)

    @classmethod
    def appendLinList(self, x):
        self.__linList.append(x)

    @classmethod
    def appendPlaList(self, x):
        self.__plaList.append(x)

    @classmethod
    def removeFromObjDict(self, x):
        # Return erlaubt das Löschen von Objekten im Format:
        # ObjectLists.removeFrom???List(ObjectLists.removeFromObjDict("objID"))
        return self.__objDict.pop(x)

    @classmethod
    def removeFromVecList(self, x):
        self.__vecList.remove(x)

    @classmethod
    def removeFromPoiList(self, x):
        self.__poiList.remove(x)

    @classmethod
    def removeFromLinList(self, x):
        self.__linList.remove(x)

    @classmethod
    def removeFromPlaList(self, x):
        self.__plaList.remove(x)



import math
from typing import Any
from listclass import ObjectLists
from vector3Dclass import Vector3D as Vector3D
from colorAssignclass import ColorAssign
from nameAssignclass import NameAssign
from pointclass import Point


class Line:
    __idTag = "lin"
    __idCount = 0

    def __init__(self,
                 positionVector: Vector3D,
                 directionVector: Vector3D,
                 name=None,
                 color=None,
                 show=True,
                 append=False):
        """The init method of the line class.
           Takes positionVector and directionVector as Vector3D, parameter as int or float, name as string
           and color as tuple of format: (Red, Green, Blue)(Valuerange = 0 to 256).
           Pass dtype None for color  or name for it to be automatically assigned."""
        self.__positionVector = positionVector
        self.__directionVector = directionVector
        self.show = show
        if name is None:
            self.__name = NameAssign.getNewName()
        else:
            self.__name = name
        if color is None:
            self.__color = ColorAssign.getNewColor()
        else:
            self.__color = color
        Line.__idCount += 1
        self.__id = str(self.__idTag) + str(self.__idCount)
        if append:
            ObjectLists.appendObjDict({str(self.__id): self})
            ObjectLists.appendLinList(self)


    def __del__(self):
        ColorAssign.removeColor(self.__color)
        NameAssign.removeName(self.__name)
        try:
            ObjectLists.removeFromLinList(self)
            ObjectLists.removeFromObjDict(str(self.__id))
        except ValueError:
            pass

    def getPositionVector(self):
        """The get method for the positionVector. Returns the positionVector as type Vector3D."""
        return self.__positionVector

    def getDirectionVector(self):
        """The get method for the driectionVector. Returns the directionVector as type Vector3D."""
        return self.__directionVector

    def getName(self):
        """The get method for the name. Returns name as a string."""
        return str(self.__name)

    def getID(self):
        """The get method for the ID. Returns ID as string."""
        return str(self.__id)

    def getColor(self):
        """The get method for the color.
           Returns color as tuple of format: (Red, Green, Blue). Valuerange = 0 to 256"""
        return self.__color

    def setPositionVector(self, posVector):
        """The set method for the positionVector.
           Takes the positionVector as type Vector3D."""
        self.__positionVector = posVector

    def setPositionVectorX(self, x):
        """The set method for the x component of the positionVector.
           Takes the x component of the positionVector as float."""
        self.__positionVector.setX(x)

    def setPositionVectorY(self, y):
        """The set method for the y component of the positionVector.
           Takes the y component of the positionVector as float."""
        self.__positionVector.setY(float(y))

    def setPositionVectorZ(self, z):
        """The set method for the z component of the positionVector.
           Takes the z component of the positionVector as float."""
        self.__positionVector.setZ(z)

    def setDirectionVector(self, dirVector):
        """The set method for the driectionVector.
           Takes the directionVector as type Vector3D."""
        self.__directionVector = dirVector

    def setDirectionVectorX(self, x):
        """The set method for the x component of the directionVector.
           Takes the x component of the directionVector as float."""
        self.__directionVector.setX(x)

    def setDirectionVectorY(self, y):
        """The set method for the y component of the directionVector.
           Takes the y component of the directionVector as float."""
        self.__directionVector.setY(y)

    def setDirectionVectorZ(self, z):
        """The set method for the z component of the directionVector.
           Takes the z component of the directionVector as float."""
        self.__directionVector.setZ(z)

    def setName(self, name):
        """The set method for the name. Takes name as a string."""
        self.__name = str(name)

    def setColor(self, color):
        """The set method for the color. Takes color as tuple of format: (Red, Green, Blue). Valuerange = 0 to 256"""
        self.__color = color

    def pointOnLine(self, coefficient):
        """Returns a Point objekt of the point on the line with the coefficient.
           Takes coefficient as float or int."""
        return Point(
            self.__directionVector.x*coefficient+self.__positionVector.x,
            self.__directionVector.y*coefficient+self.__positionVector.y,
            self.__directionVector.z*coefficient+self.__positionVector.z
            )

    def normalizeDirectionVector(self):
        """Not yet implemented. Should make the length of the direction vector equal to 1,
           by dividing the vector with its length. (Einheitsnormalvektor)"""
        pass

    def __str__(self):
        """Str method. Using given line instance, returns string."""
        return "x = "+ str(self.__positionVector)+" + r * "+str(self.__directionVector)

    posVec = property(getPositionVector, setPositionVector)

    dirVec = property(getDirectionVector, setDirectionVector)

    name = property(getName, setName)




from typing import Any
from math import sqrt
from listclass import ObjectLists
from vector3Dclass import Vector3D as Vector3D
from nameAssignclass import NameAssign
from colorAssignclass import ColorAssign
import numpy as np


class Plane:
    __idTag = "pla"
    __idCount = 0

    @classmethod
    def parameterForm(cls,
                      positionVector: Vector3D,
                      directionVectorOne: Vector3D,
                      directionVectorTwo: Vector3D,
                      name=None,
                      color=None,
                      show=True,
                      append=False):
        """The classmethod for initializing a Plane in parameter Form.
           Takes positionVector and directionVectorOne and directionVectorTwo as Vector3D, parameter as int or float,
           name as string and color as tuple of format: (Red, Green, Blue)(Valuerange = 0 to 256)."""
        plane = cls(positionVector, directionVectorOne, directionVectorTwo, None, None, name, color, show, append, typeOfPlane="parameter")
        return plane

    @classmethod
    def normalForm(cls,
                   positionVector: Vector3D,
                   normalVector: Vector3D,
                   name=None,
                   color=None,
                   show=True,
                   append=False):
        """The classmethod for intializing a Plane in normal Form.
           Takes positionVector and normalVector as Vector3D, name as string and color as tuple of format:
           (Red, Green, Blue)(Valuerange = 0 to 256)."""
        plane = cls(positionVector, None, None, normalVector, None, name, color, show, append, typeOfPlane="normal")
        return plane

    @classmethod
    def coordinateForm(cls,
                       normalVector :Vector3D,
                       scalarParameter :float,
                       name=None,
                       color=None,
                       show=True,
                       append=False):
        """The classmethod for initializing a Plane in coordinate Form.
           Takes a, b, c, d as float, name as string and color as tuple of format:
           (Red, Green, Blue)(Valuerange = 0 to 256).
           For a equation of the form: ax + by + cz = d."""
        plane = cls(None, None, None, normalVector, scalarParameter, name, color, show, append, typeOfPlane="coordinate")
        return plane

    def __init__(self,
                 positionVector: Vector3D,
                 directionVectorOne: Vector3D,
                 directionVectorTwo: Vector3D,
                 normalVector: Vector3D,
                 scalarParameter: float,
                 name=None,
                 color=None,
                 show=True,
                 append=False,
                 typeOfPlane=""):
        """The init method of the plane class.
           Takes positionVector and directionVectorOne and directionVectorTwo as Vector3D, parameter as int or float,
           name as string and color as tuple of format: (Red, Green, Blue)(Valuerange = 0 to 256)."""
        self.__positionVector = positionVector
        self.__directionVectorOne = directionVectorOne
        self.__directionVectorTwo = directionVectorTwo
        self.__normalVector = normalVector
        self.show = show
        self.append = append
        self.__typeOfPlane = typeOfPlane
        # Following code checks if a normalVector was passed. If not: generates normalVector from directionVectors.

        try:
            self.__scalarParameter = scalarParameter
        except Exception:
            pass

        if name is None:
            self.__name = NameAssign.getNewName()
        else:
            self.__name = name
        if color is None:
            self.__color = ColorAssign.getNewColor()
        else:
            self.__color = color
        Plane.__idCount += 1
        self.__id = str(self.__idTag) + str(self.__idCount)
        if append:
            ObjectLists.appendObjDict({str(self.__id): self})
            ObjectLists.appendPlaList(self)


    def __del__(self):
        ColorAssign.removeColor(self.__color)
        NameAssign.removeName(self.__name)
        try:
            ObjectLists.removeFromPlaList(self)
            ObjectLists.removeFromObjDict(str(self.__id))
        except ValueError:
            pass

    def getPositionVector(self):
        """The get method for the positionVector. Returns the positionVector as type Vector3D."""
        return self.__positionVector

    def getDirectionVectorOne(self):
        """The get method for the driectionVectorOne. Returns the directionVectorOne as type Vector3D."""
        return self.__directionVectorOne

    def getDirectionVectorTwo(self):
        """The get method for the driectionVectorTwo. Returns the directionVectorTwo as type Vector3D."""
        return self.__directionVectorTwo

    def getNormalVector(self):
        """The get method for the normalVector. Returns normalVector as Vector3D."""
        return self.__normalVector

    def getScalarParameter(self):
        """The get method for the scalarParameter. Returns scalarParameter as float."""
        return float(self.__scalarParameter)

    def getName(self):
        """The get method for the name. Returns name as a string."""
        return str(self.__name)

    def getID(self):
        """The get method for the ID. Returns ID as string."""
        return str(self.__id)

    def getColor(self):
        """The get method for the color. Returns color as tuple of format: (Red, Green, Blue). Valuerange = 0 to 256"""
        return self.__color

    def getType(self):
        """The get method for the Type. Returns the type as a string."""
        return self.__typeOfPlane

    def setPositionVector(self,
                          posVector):
        """The set method for the positionVector. Takes the positionVector as type Vector3D."""
        self.__positionVector = posVector

    def setDirectionVectorOne(self,
                              directionVectorOne):
        """The set method for the driectionVectorOne. Takes the directionVectorOne as type Vector3D."""
        self.__directionVectorOne = directionVectorOne

    def setDirectionVectorTwo(self,
                              directionVectorTwo):
        """The set method for the driectionVectorTwo. Takes the directionVectorTwo as type Vector3D."""
        self.__directionVectorTwo = directionVectorTwo

    def setNormalVector(self,
                        normalVector):
        """The set method for the normalVector. Takes normalVector as type Vector3D."""
        self.__normalVector = normalVector

    def setScalarParameter(self,
                           scalarParameter):
        """The set method for the scalarParameter. Takes scalarParameter as int or float."""
        self.__scalarParameter = float(scalarParameter)

    def setName(self,
                name):
        """The set method for the name. Takes name as a string."""
        self.__name = str(name)


    def setColor(self,
                 color):
        """The set method for the color. Takes color as tuple of format: (Red, Green, Blue). Valuerange = 0 to 256"""
        self.__color = color
    
    def getType(self):
        return self.__typeOfPlane

    def convertToHessianNormalForm(self):
        """Converts the given Instance to Hessian Normal Form.
        The Normal Vector is transformed into a Normal Vector of length 1."""
        if self.__typeOfPlane == "parameter":
            posVec = self.__positionVector
            normVec = self.getDirectionVectorOne().vectorProduct(self.getDirectionVectorTwo())
            nameVec = self.__name
            hess = Plane.normalForm(posVec, normVec, nameVec)
            hess.setNormalVector(normVec.scalarDivision(normVec.length()))
            self.__typeOfPlane = "normal"
        elif self.__typeOfPlane == "coordinate":
            normVec = self.getNormalVector().scalarDivision(self.__normalVector.length())
            scalParam = self.getScalarParameter()/self.getNormalVector().length()
            hess = Plane.coordinateForm(normVec,scalParam)
        elif self.__typeOfPlane == "normal":
            normVec = self.getNormalVector().scalarDivision(self.getNormalVector().length())
            hess = Plane.normalForm(self.getPositionVector(),normVec)
        return hess  

    def __str__(self):
        if self.__typeOfPlane == "normal":
            return "(x - "+ str(self.__positionVector)+" + ) * "+str(self.__normalVector)+" = 0"
        elif self.__typeOfPlane == "parameter":
            return "x = "+str(self.__positionVector)+" + r * "+str(self.__directionVectorOne)+" + s * "+str(self.__directionVectorTwo)
        elif self.__typeOfPlane == "coordinate":
            return +str(self.__normalVector.getX())+"x + "+str(self.__normalVector.getY())+"y + "+str(self.__normalVector.getZ())+"z = "+str(self.__scalarParameter)
        else:
            return "No valid plane!"

    posVec = property(getPositionVector, setPositionVector)

    normVec = property(getNormalVector, setNormalVector)

    dirVecOne = property(getDirectionVectorOne, setDirectionVectorOne)

    dirVecTwo = property(getDirectionVectorTwo, setDirectionVectorTwo)

    scalParam = property(getScalarParameter, setScalarParameter)

    name = property(getName, setName)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vektorplotter']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.4,<6.0.0',
 'PyQtWebEngine>=5.15.4,<6.0.0',
 'numpy>=1.20.1,<2.0.0',
 'pandas>=1.2.1,<2.0.0',
 'plotly>=4.14.3,<5.0.0']

setup_kwargs = {
    'name': 'vektorplotter',
    'version': '0.1.0',
    'description': 'A simple program for basic features of analytical geometry.',
    'long_description': None,
    'author': 'Benjamin Brumm',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
